@@ -9,7 +9,9 @@ from enum import Enum
 assert typing
 
 
-def normalize_to_json_compat(val: Any) -> typing.Dict[Any, Any]:
+def normalize_to_json_compat(val: Any) -> Any:
+    if val is None:
+        return None
     if hasattr(val, '__dataclass_fields__'):
         json_data = dataclasses.asdict(val)
         for key in json_data.keys():
@@ -22,7 +24,7 @@ def normalize_to_json_compat(val: Any) -> typing.Dict[Any, Any]:
     elif type(val) in [dict] or (getattr(type(val), '_name', None) in ('Dict', 'Mapping')):
         json_data = {}
         for key, value in val.items():
-            json_data[key] = normalize_to_json_compat(value)
+            json_data[to_str(key)] = normalize_to_json_compat(value)
     elif type(val) in [list] or (getattr(type(val), '_name', None) in ('List', )):
         json_data = []
         for value in val:
@@ -32,11 +34,19 @@ def normalize_to_json_compat(val: Any) -> typing.Dict[Any, Any]:
     return json_data
 
 
-def normalize_from_json(json_data, typ):
+def normalize_from_json(json_data, typ) -> Any:
+    if typ is None or json_data is None:
+        return None
     if hasattr(typ, '_name') and (str(typ).startswith('typing.Union') or str(typ).startswith('typing.Optional')):
-        typ = typ.__args__[0]
-        if str(typ).startswith('typing.Optional') and json_data == 'null':
-            return None
+        for arg in typ.__args__:
+            if arg in (str, int, float, ) and type(json_data) == arg:
+                return json_data
+            elif arg in (str, int, float):
+                continue
+            try:
+                return normalize_from_json(json_data, arg)
+            except Exception:
+                continue
     if _issubclass_safe(typ, Enum):
         for key in typ.__members__:
             t = type(typ.__members__[key].value)
@@ -54,20 +64,25 @@ def normalize_from_json(json_data, typ):
     elif typ == bool:
         return json_data.lower() == 'true'
     elif getattr(typ, '_name', None) in ('Dict', 'Mapping'):
-        key_typ, elem_typ = eval(str(typ).split('[', maxsplit=1)[1][:-1])
-        d = dict(json_data)
-        for k, v in json_data.items():
-            d[normalize_from_json(k, key_typ)] = normalize_from_json(v, elem_typ)
-        return d
+        key_typ, elem_typ = typ.__args__
+        return {
+            normalize_from_json(k, key_typ):  normalize_from_json(v, elem_typ)
+            for k, v in json_data.items()
+        }
     elif getattr(typ, '_name', None) in ('List', ):
-        elem_typ = eval(str(typ).split('[', maxsplit=1)[1][:-1])
-        for index, value in enumerate(json_data):
-            json_data[index] = normalize_from_json(value, elem_typ)
-        return json_data
+        elem_typ = typ.__args__[0]
+        return [normalize_from_json(value, elem_typ) for value in json_data]
+    elif getattr(typ, '_name', None) in ('Set', ):
+        elem_typ = typ.__args__[0]
+        return {normalize_from_json(value, elem_typ) for value in json_data}
+    elif getattr(typ, '_name', None) in ('Tuple', ):
+        elem_typ = typ.__args__[0]
+        return tuple(normalize_from_json(value, elem_typ) for value in json_data)
     elif hasattr(typ, '__dataclass_fields__'):
-        for name, field in typ.__dataclass_fields__.items():
-            json_data[name] = normalize_from_json(json_data[name], field.type)
-        return typ(**json_data)
+        return typ(**{
+            name: normalize_from_json(json_data[name], field.type)
+            for name, field in typ.__dataclass_fields__.items()
+        })
     else:
         raise TypeError(f"Unsupported typ for web api: '{typ}'")
 
@@ -80,14 +95,19 @@ def to_str(val: Any) -> Optional[str]:
         return json.dumps(val)
     elif isinstance(val, Enum):
         return val.value
-    elif type(val) in (str, int, float, bool):
-        return val
+    elif type(val) == bool:
+        return str(val).lower()
+    elif type(val) in (str, int, float):
+        return str(val)
     elif type(val) in [dict] or (getattr(type(val), '_name', None) in ('Dict', 'Mapping')):
         val = normalize_to_json_compat(val)
         return json.dumps(val)
     elif type(val) in [list] or (getattr(type(val), '_name', None) in ('List', )):
         val = normalize_to_json_compat(val)
         return json.dumps(val)
+    elif type(val) in [set, tuple] or (getattr(type(val), '_name', None)) in ('Set', 'Tuple', ):
+        val = normalize_to_json_compat(list(val))
+        return json.dumps(val)
     raise TypeError(f"Type of value, '{type(val)}' is not supported in web api")
 
 
@@ -112,7 +132,9 @@ def from_str(image: str, typ: Type) -> Any:
         return typ(image)
     elif typ == bool:
         return image.lower() == 'true'
-    elif typ in (dict, list) or (getattr(typ, '_name', None) in ('Dict', 'List', 'Mapping')):
+    elif getattr(typ, '_name', None) in ('Dict', 'List', 'Mapping'):
+        return normalize_from_json(json.loads(image), typ)
+    elif getattr(typ, '_name', None) in ('Set', 'Tuple'):
         return normalize_from_json(json.loads(image), typ)
     elif hasattr(typ, '__dataclass_fields__'):
         return normalize_from_json(json.loads(image), typ)
